@@ -4,7 +4,7 @@
  *                Daniel R. Reynolds @ SMU
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2022, Lawrence Livermore National Security
+ * Copyright (c) 2002-2024, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -20,14 +20,21 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <R_ext/Print.h>
+#define fprintf(outf,...) Rprintf(__VA_ARGS__)
 
+#include <sundials/priv/sundials_context_impl.h>
+#include <sundials/sundials_core.h>
 #include <sundials/sundials_nvector.h>
-#include "sundials_context_impl.h"
+
+#include "sundials/priv/sundials_errors_impl.h"
+#include "sundials/sundials_errors.h"
+#include "sundials/sundials_types.h"
 
 #if defined(SUNDIALS_BUILD_WITH_PROFILING)
-static SUNProfiler getSUNProfiler(N_Vector v)
+static inline SUNProfiler getSUNProfiler(N_Vector v)
 {
-  return(v->sunctx->profiler);
+  return (v->sunctx->profiler);
 }
 #endif
 
@@ -38,20 +45,21 @@ static SUNProfiler getSUNProfiler(N_Vector v)
 /* Create an empty NVector object */
 N_Vector N_VNewEmpty(SUNContext sunctx)
 {
-  N_Vector     v;
-  N_Vector_Ops ops;
+  if (sunctx == NULL) { return NULL; }
 
-  if (sunctx == NULL) return(NULL);
+  SUNFunctionBegin(sunctx);
+  N_Vector v;
+  N_Vector_Ops ops;
 
   /* create vector object */
   v = NULL;
-  v = (N_Vector) malloc(sizeof *v);
-  if (v == NULL) return(NULL);
+  v = (N_Vector)malloc(sizeof *v);
+  SUNAssertNull(v, SUN_ERR_MALLOC_FAIL);
 
   /* create vector ops structure */
   ops = NULL;
-  ops = (N_Vector_Ops) malloc(sizeof *ops);
-  if (ops == NULL) { free(v); return(NULL); }
+  ops = (N_Vector_Ops)malloc(sizeof *ops);
+  SUNAssertNull(ops, SUN_ERR_MALLOC_FAIL);
 
   /* initialize operations to NULL */
 
@@ -97,7 +105,7 @@ N_Vector N_VNewEmpty(SUNContext sunctx)
   /*
    * OPTIONAL operations.
    *
-   * These operations provide default implementations that may be overriden.
+   * These operations provide default implementations that may be overridden.
    */
 
   /* fused vector operations (optional) */
@@ -118,18 +126,18 @@ N_Vector N_VNewEmpty(SUNContext sunctx)
    * OPTIONAL operations with no default implementation.
    */
 
-  ops->nvgetlocallength    = NULL;
+  ops->nvgetlocallength = NULL;
 
   /* local reduction operations (optional) */
-  ops->nvdotprodlocal      = NULL;
-  ops->nvmaxnormlocal      = NULL;
-  ops->nvminlocal          = NULL;
-  ops->nvl1normlocal       = NULL;
-  ops->nvinvtestlocal      = NULL;
-  ops->nvconstrmasklocal   = NULL;
-  ops->nvminquotientlocal  = NULL;
-  ops->nvwsqrsumlocal      = NULL;
-  ops->nvwsqrsummasklocal  = NULL;
+  ops->nvdotprodlocal     = NULL;
+  ops->nvmaxnormlocal     = NULL;
+  ops->nvminlocal         = NULL;
+  ops->nvl1normlocal      = NULL;
+  ops->nvinvtestlocal     = NULL;
+  ops->nvconstrmasklocal  = NULL;
+  ops->nvminquotientlocal = NULL;
+  ops->nvwsqrsumlocal     = NULL;
+  ops->nvwsqrsummasklocal = NULL;
 
   /* single buffer reduction operations */
   ops->nvdotprodmultilocal     = NULL;
@@ -151,29 +159,30 @@ N_Vector N_VNewEmpty(SUNContext sunctx)
   v->content = NULL;
   v->sunctx  = sunctx;
 
-  return(v);
+  return v;
 }
 
 /* Free a generic N_Vector (assumes content is already empty) */
 void N_VFreeEmpty(N_Vector v)
 {
-  if (v == NULL)  return;
+  if (v == NULL) { return; }
 
   /* free non-NULL ops structure */
-  if (v->ops)  free(v->ops);
+  if (v->ops) { free(v->ops); }
   v->ops = NULL;
 
   /* free overall N_Vector object and return */
   free(v);
+
   return;
 }
 
 /* Copy a vector 'ops' structure */
-int N_VCopyOps(N_Vector w, N_Vector v)
+SUNErrCode N_VCopyOps(N_Vector w, N_Vector v)
 {
+  SUNFunctionBegin(w->sunctx);
   /* Check that ops structures exist */
-  if (w == NULL || v == NULL) return(-1);
-  if (w->ops == NULL || v->ops == NULL) return(-1);
+  SUNAssert(w && w->ops && v && v->ops, SUN_ERR_ARG_CORRUPT);
 
   /* Copy ops from w to v */
 
@@ -220,7 +229,7 @@ int N_VCopyOps(N_Vector w, N_Vector v)
   /*
    * OPTIONAL operations.
    *
-   * These operations provide default implementations that may be overriden.
+   * These operations provide default implementations that may be overridden.
    */
 
   /* fused vector operations */
@@ -229,12 +238,12 @@ int N_VCopyOps(N_Vector w, N_Vector v)
   v->ops->nvdotprodmulti      = w->ops->nvdotprodmulti;
 
   /* vector array operations */
-  v->ops->nvlinearsumvectorarray         = w->ops->nvlinearsumvectorarray;
-  v->ops->nvscalevectorarray             = w->ops->nvscalevectorarray;
-  v->ops->nvconstvectorarray             = w->ops->nvconstvectorarray;
-  v->ops->nvwrmsnormvectorarray          = w->ops->nvwrmsnormvectorarray;
-  v->ops->nvwrmsnormmaskvectorarray      = w->ops->nvwrmsnormmaskvectorarray;
-  v->ops->nvscaleaddmultivectorarray     = w->ops->nvscaleaddmultivectorarray;
+  v->ops->nvlinearsumvectorarray     = w->ops->nvlinearsumvectorarray;
+  v->ops->nvscalevectorarray         = w->ops->nvscalevectorarray;
+  v->ops->nvconstvectorarray         = w->ops->nvconstvectorarray;
+  v->ops->nvwrmsnormvectorarray      = w->ops->nvwrmsnormvectorarray;
+  v->ops->nvwrmsnormmaskvectorarray  = w->ops->nvwrmsnormmaskvectorarray;
+  v->ops->nvscaleaddmultivectorarray = w->ops->nvscaleaddmultivectorarray;
   v->ops->nvlinearcombinationvectorarray = w->ops->nvlinearcombinationvectorarray;
 
   /*
@@ -242,15 +251,15 @@ int N_VCopyOps(N_Vector w, N_Vector v)
    */
 
   /* local reduction operations */
-  v->ops->nvdotprodlocal      = w->ops->nvdotprodlocal;
-  v->ops->nvmaxnormlocal      = w->ops->nvmaxnormlocal;
-  v->ops->nvminlocal          = w->ops->nvminlocal;
-  v->ops->nvl1normlocal       = w->ops->nvl1normlocal;
-  v->ops->nvinvtestlocal      = w->ops->nvinvtestlocal;
-  v->ops->nvconstrmasklocal   = w->ops->nvconstrmasklocal;
-  v->ops->nvminquotientlocal  = w->ops->nvminquotientlocal;
-  v->ops->nvwsqrsumlocal      = w->ops->nvwsqrsumlocal;
-  v->ops->nvwsqrsummasklocal  = w->ops->nvwsqrsummasklocal;
+  v->ops->nvdotprodlocal     = w->ops->nvdotprodlocal;
+  v->ops->nvmaxnormlocal     = w->ops->nvmaxnormlocal;
+  v->ops->nvminlocal         = w->ops->nvminlocal;
+  v->ops->nvl1normlocal      = w->ops->nvl1normlocal;
+  v->ops->nvinvtestlocal     = w->ops->nvinvtestlocal;
+  v->ops->nvconstrmasklocal  = w->ops->nvconstrmasklocal;
+  v->ops->nvminquotientlocal = w->ops->nvminquotientlocal;
+  v->ops->nvwsqrsumlocal     = w->ops->nvwsqrsumlocal;
+  v->ops->nvwsqrsummasklocal = w->ops->nvwsqrsummasklocal;
 
   /* single buffer reduction operations */
   v->ops->nvdotprodmultilocal     = w->ops->nvdotprodmultilocal;
@@ -265,24 +274,21 @@ int N_VCopyOps(N_Vector w, N_Vector v)
   v->ops->nvprint     = w->ops->nvprint;
   v->ops->nvprintfile = w->ops->nvprintfile;
 
-  return(0);
+  return SUN_SUCCESS;
 }
 
 /* -----------------------------------------------------------------
  * Functions in the 'ops' structure
  * -----------------------------------------------------------------*/
 
-N_Vector_ID N_VGetVectorID(N_Vector w)
-{
-  return(w->ops->nvgetvectorid(w));
-}
+N_Vector_ID N_VGetVectorID(N_Vector w) { return (w->ops->nvgetvectorid(w)); }
 
 N_Vector N_VClone(N_Vector w)
 {
   N_Vector result = NULL;
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(w));
   result = w->ops->nvclone(w);
-  result->sunctx = w->sunctx;
+  if (result) { result->sunctx = w->sunctx; }
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(w));
   return result;
 }
@@ -292,76 +298,89 @@ N_Vector N_VCloneEmpty(N_Vector w)
   N_Vector result;
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(w));
   result = w->ops->nvcloneempty(w);
-  result->sunctx = w->sunctx;
+  if (result) { result->sunctx = w->sunctx; }
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(w));
   return result;
 }
 
 void N_VDestroy(N_Vector v)
 {
-  if (v == NULL) return;
+  if (v == NULL) { return; }
 
   /* if the destroy operation exists use it */
-  if (v->ops)
-    if (v->ops->nvdestroy) { v->ops->nvdestroy(v); return; }
-
-  /* if we reach this point, either ops == NULL or nvdestroy == NULL,
-     try to cleanup by freeing the content, ops, and vector */
-  if (v->content) { free(v->content); v->content = NULL; }
-  if (v->ops) { free(v->ops); v->ops = NULL; }
-  free(v); v = NULL;
+  if (v->ops->nvdestroy) { v->ops->nvdestroy(v); }
+  else
+  {
+    /* if we reach this point, either ops == NULL or nvdestroy == NULL,
+      try to cleanup by freeing the content, ops, and vector */
+    if (v->content)
+    {
+      free(v->content);
+      v->content = NULL;
+    }
+    if (v->ops)
+    {
+      free(v->ops);
+      v->ops = NULL;
+    }
+    free(v);
+    v = NULL;
+  }
 
   return;
 }
 
-void N_VSpace(N_Vector v, sunindextype *lrw, sunindextype *liw)
+void N_VSpace(N_Vector v, sunindextype* lrw, sunindextype* liw)
 {
   v->ops->nvspace(v, lrw, liw);
   return;
 }
 
-realtype *N_VGetArrayPointer(N_Vector v)
+sunrealtype* N_VGetArrayPointer(N_Vector v)
 {
-  return((realtype *) v->ops->nvgetarraypointer(v));
+  if (v->ops->nvgetarraypointer)
+  {
+    return (sunrealtype*)v->ops->nvgetarraypointer(v);
+  }
+  else { return NULL; }
 }
 
-realtype *N_VGetDeviceArrayPointer(N_Vector v)
+sunrealtype* N_VGetDeviceArrayPointer(N_Vector v)
 {
   if (v->ops->nvgetdevicearraypointer)
-    return((realtype *) v->ops->nvgetdevicearraypointer(v));
-  else
-    return(NULL);
+  {
+    return ((sunrealtype*)v->ops->nvgetdevicearraypointer(v));
+  }
+  else { return (NULL); }
 }
 
-void N_VSetArrayPointer(realtype *v_data, N_Vector v)
+void N_VSetArrayPointer(sunrealtype* v_data, N_Vector v)
 {
-  v->ops->nvsetarraypointer(v_data, v);
+  if (v->ops->nvsetarraypointer) { v->ops->nvsetarraypointer(v_data, v); }
   return;
 }
 
-void *N_VGetCommunicator(N_Vector v)
+SUNComm N_VGetCommunicator(N_Vector v)
 {
-  if (v->ops->nvgetcommunicator)
-    return(v->ops->nvgetcommunicator(v));
-  else
-    return(NULL);
+  if (v->ops->nvgetcommunicator) { return (v->ops->nvgetcommunicator(v)); }
+  else { return (SUN_COMM_NULL); }
 }
 
 sunindextype N_VGetLength(N_Vector v)
 {
-  return((sunindextype) v->ops->nvgetlength(v));
+  return ((sunindextype)v->ops->nvgetlength(v));
 }
 
 sunindextype N_VGetLocalLength(N_Vector v)
 {
-  return((sunindextype) v->ops->nvgetlocallength(v));
+  return ((sunindextype)v->ops->nvgetlocallength(v));
 }
 
 /* -----------------------------------------------------------------
  * standard vector operations
  * -----------------------------------------------------------------*/
 
-void N_VLinearSum(realtype a, N_Vector x, realtype b, N_Vector y, N_Vector z)
+void N_VLinearSum(sunrealtype a, N_Vector x, sunrealtype b, N_Vector y, N_Vector z)
 {
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
   z->ops->nvlinearsum(a, x, b, y, z);
@@ -369,7 +388,7 @@ void N_VLinearSum(realtype a, N_Vector x, realtype b, N_Vector y, N_Vector z)
   return;
 }
 
-void N_VConst(realtype c, N_Vector z)
+void N_VConst(sunrealtype c, N_Vector z)
 {
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(z));
   z->ops->nvconst(c, z);
@@ -393,7 +412,7 @@ void N_VDiv(N_Vector x, N_Vector y, N_Vector z)
   return;
 }
 
-void N_VScale(realtype c, N_Vector x, N_Vector z)
+void N_VScale(sunrealtype c, N_Vector x, N_Vector z)
 {
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
   z->ops->nvscale(c, x, z);
@@ -417,7 +436,7 @@ void N_VInv(N_Vector x, N_Vector z)
   return;
 }
 
-void N_VAddConst(N_Vector x, realtype b, N_Vector z)
+void N_VAddConst(N_Vector x, sunrealtype b, N_Vector z)
 {
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
   z->ops->nvaddconst(x, b, z);
@@ -425,70 +444,70 @@ void N_VAddConst(N_Vector x, realtype b, N_Vector z)
   return;
 }
 
-realtype N_VDotProd(N_Vector x, N_Vector y)
+sunrealtype N_VDotProd(N_Vector x, N_Vector y)
 {
-  realtype result;
+  sunrealtype result;
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
-  result = ((realtype) y->ops->nvdotprod(x, y));
+  result = ((sunrealtype)y->ops->nvdotprod(x, y));
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
-  return(result);
+  return (result);
 }
 
-realtype N_VMaxNorm(N_Vector x)
+sunrealtype N_VMaxNorm(N_Vector x)
 {
-  realtype result;
+  sunrealtype result;
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
-  result = ((realtype) x->ops->nvmaxnorm(x));
+  result = ((sunrealtype)x->ops->nvmaxnorm(x));
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
-  return(result);
+  return (result);
 }
 
-realtype N_VWrmsNorm(N_Vector x, N_Vector w)
+sunrealtype N_VWrmsNorm(N_Vector x, N_Vector w)
 {
-  realtype result;
+  sunrealtype result;
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
-  result = ((realtype) x->ops->nvwrmsnorm(x, w));
+  result = ((sunrealtype)x->ops->nvwrmsnorm(x, w));
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
-  return(result);
+  return (result);
 }
 
-realtype N_VWrmsNormMask(N_Vector x, N_Vector w, N_Vector id)
+sunrealtype N_VWrmsNormMask(N_Vector x, N_Vector w, N_Vector id)
 {
-  realtype result;
+  sunrealtype result;
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
-  result = ((realtype) x->ops->nvwrmsnormmask(x, w, id));
+  result = ((sunrealtype)x->ops->nvwrmsnormmask(x, w, id));
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
-  return(result);
+  return (result);
 }
 
-realtype N_VMin(N_Vector x)
+sunrealtype N_VMin(N_Vector x)
 {
-  realtype result;
+  sunrealtype result;
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
-  result = ((realtype) x->ops->nvmin(x));
+  result = ((sunrealtype)x->ops->nvmin(x));
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
-  return(result);
+  return (result);
 }
 
-realtype N_VWL2Norm(N_Vector x, N_Vector w)
+sunrealtype N_VWL2Norm(N_Vector x, N_Vector w)
 {
-  realtype result;
+  sunrealtype result;
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
-  result = ((realtype) x->ops->nvwl2norm(x, w));
+  result = ((sunrealtype)x->ops->nvwl2norm(x, w));
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
-  return(result);
+  return (result);
 }
 
-realtype N_VL1Norm(N_Vector x)
+sunrealtype N_VL1Norm(N_Vector x)
 {
-  realtype result;
+  sunrealtype result;
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
-  result = ((realtype) x->ops->nvl1norm(x));
+  result = ((sunrealtype)x->ops->nvl1norm(x));
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
-  return(result);
+  return (result);
 }
 
-void N_VCompare(realtype c, N_Vector x, N_Vector z)
+void N_VCompare(sunrealtype c, N_Vector x, N_Vector z)
 {
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
   z->ops->nvcompare(c, x, z);
@@ -496,468 +515,486 @@ void N_VCompare(realtype c, N_Vector x, N_Vector z)
   return;
 }
 
-booleantype N_VInvTest(N_Vector x, N_Vector z)
+sunbooleantype N_VInvTest(N_Vector x, N_Vector z)
 {
-  booleantype result;
+  sunbooleantype result;
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
-  result = ((booleantype) z->ops->nvinvtest(x, z));
+  result = ((sunbooleantype)z->ops->nvinvtest(x, z));
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
-  return(result);
+  return (result);
 }
 
-booleantype N_VConstrMask(N_Vector c, N_Vector x, N_Vector m)
+sunbooleantype N_VConstrMask(N_Vector c, N_Vector x, N_Vector m)
 {
-  booleantype result;
+  sunbooleantype result;
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(c));
-  result = ((booleantype) x->ops->nvconstrmask(c, x, m));
+  result = ((sunbooleantype)x->ops->nvconstrmask(c, x, m));
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(c));
-  return(result);
+  return (result);
 }
 
-realtype N_VMinQuotient(N_Vector num, N_Vector denom)
+sunrealtype N_VMinQuotient(N_Vector num, N_Vector denom)
 {
-  realtype result;
+  sunrealtype result;
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(num));
-  result = ((realtype) num->ops->nvminquotient(num, denom));
+  result = ((sunrealtype)num->ops->nvminquotient(num, denom));
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(num));
-  return(result);
+  return (result);
 }
 
-
-
 /* -----------------------------------------------------------------
  * OPTIONAL fused vector operations
  * -----------------------------------------------------------------*/
 
-int N_VLinearCombination(int nvec, realtype* c, N_Vector* X, N_Vector z)
+SUNErrCode N_VLinearCombination(int nvec, sunrealtype* c, N_Vector* X, N_Vector z)
 {
-  int i, ier;
+  int i;
+  SUNErrCode ier;
+
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(X[0]));
 
-  if (z->ops->nvlinearcombination != NULL) {
-
+  if (z->ops->nvlinearcombination != NULL)
+  {
     ier = z->ops->nvlinearcombination(nvec, c, X, z);
-
-  } else {
-
+  }
+  else
+  {
     z->ops->nvscale(c[0], X[0], z);
-    for (i=1; i<nvec; i++) {
-      z->ops->nvlinearsum(c[i], X[i], RCONST(1.0), z, z);
+    for (i = 1; i < nvec; i++)
+    {
+      z->ops->nvlinearsum(c[i], X[i], SUN_RCONST(1.0), z, z);
     }
-    ier = 0;
-
+    ier = SUN_SUCCESS;
   }
 
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(X[0]));
-  return(ier);
+  return (ier);
 }
 
-int N_VScaleAddMulti(int nvec, realtype* a, N_Vector x, N_Vector* Y, N_Vector* Z)
+SUNErrCode N_VScaleAddMulti(int nvec, sunrealtype* a, N_Vector x, N_Vector* Y,
+                            N_Vector* Z)
 {
-  int i, ier;
+  int i;
+  SUNErrCode ier;
+
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
 
-  if (x->ops->nvscaleaddmulti != NULL) {
-
+  if (x->ops->nvscaleaddmulti != NULL)
+  {
     ier = x->ops->nvscaleaddmulti(nvec, a, x, Y, Z);
-
-  } else {
-
-    for (i=0; i<nvec; i++) {
-      x->ops->nvlinearsum(a[i], x, RCONST(1.0), Y[i], Z[i]);
+  }
+  else
+  {
+    for (i = 0; i < nvec; i++)
+    {
+      x->ops->nvlinearsum(a[i], x, SUN_RCONST(1.0), Y[i], Z[i]);
     }
-    ier = 0;
-
+    ier = SUN_SUCCESS;
   }
 
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
-  return(ier);
+  return (ier);
 }
 
-int N_VDotProdMulti(int nvec, N_Vector x, N_Vector* Y, realtype* dotprods)
+SUNErrCode N_VDotProdMulti(int nvec, N_Vector x, N_Vector* Y,
+                           sunrealtype* dotprods)
 {
-  int i, ier;
+  int i;
+  SUNErrCode ier;
+
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
 
-  if (x->ops->nvdotprodmulti != NULL) {
-
+  if (x->ops->nvdotprodmulti != NULL)
+  {
     ier = x->ops->nvdotprodmulti(nvec, x, Y, dotprods);
-
-  } else {
-
-    for (i=0; i<nvec; i++) {
-      dotprods[i] = x->ops->nvdotprod(x, Y[i]);
-    }
-    ier = 0;
-
+  }
+  else
+  {
+    for (i = 0; i < nvec; i++) { dotprods[i] = x->ops->nvdotprod(x, Y[i]); }
+    ier = SUN_SUCCESS;
   }
 
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
-  return(ier);
+  return (ier);
 }
 
 /* -----------------------------------------------------------------
  * OPTIONAL vector array operations
  * -----------------------------------------------------------------*/
 
-int N_VLinearSumVectorArray(int nvec, realtype a, N_Vector* X,
-                            realtype b, N_Vector* Y, N_Vector* Z)
+SUNErrCode N_VLinearSumVectorArray(int nvec, sunrealtype a, N_Vector* X,
+                                   sunrealtype b, N_Vector* Y, N_Vector* Z)
 {
-  int i, ier;
+  int i;
+  SUNErrCode ier;
+
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(X[0]));
 
-  if (Z[0]->ops->nvlinearsumvectorarray != NULL) {
-
+  if (Z[0]->ops->nvlinearsumvectorarray != NULL)
+  {
     ier = Z[0]->ops->nvlinearsumvectorarray(nvec, a, X, b, Y, Z);
-
-  } else {
-
-    for (i=0; i<nvec; i++) {
+  }
+  else
+  {
+    for (i = 0; i < nvec; i++)
+    {
       Z[0]->ops->nvlinearsum(a, X[i], b, Y[i], Z[i]);
     }
-    ier = 0;
-
+    ier = SUN_SUCCESS;
   }
 
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(X[0]));
-  return(ier);
+  return (ier);
 }
 
-int N_VScaleVectorArray(int nvec, realtype* c, N_Vector* X, N_Vector* Z)
+SUNErrCode N_VScaleVectorArray(int nvec, sunrealtype* c, N_Vector* X, N_Vector* Z)
 {
-  int i, ier;
+  int i;
+  SUNErrCode ier;
+
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(X[0]));
 
-  if (Z[0]->ops->nvscalevectorarray != NULL) {
-
+  if (Z[0]->ops->nvscalevectorarray != NULL)
+  {
     ier = Z[0]->ops->nvscalevectorarray(nvec, c, X, Z);
-
-  } else {
-
-    for (i=0; i<nvec; i++) {
-      Z[0]->ops->nvscale(c[i], X[i], Z[i]);
-    }
-    ier = 0;
-
+  }
+  else
+  {
+    for (i = 0; i < nvec; i++) { Z[0]->ops->nvscale(c[i], X[i], Z[i]); }
+    ier = SUN_SUCCESS;
   }
 
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(X[0]));
-  return(ier);
+  return (ier);
 }
 
-int N_VConstVectorArray(int nvec, realtype c, N_Vector* Z)
+SUNErrCode N_VConstVectorArray(int nvec, sunrealtype c, N_Vector* Z)
 {
-  int i, ier;
+  int i;
+  SUNErrCode ier;
+
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(Z[0]));
 
-  if (Z[0]->ops->nvconstvectorarray != NULL) {
-
+  if (Z[0]->ops->nvconstvectorarray != NULL)
+  {
     ier = Z[0]->ops->nvconstvectorarray(nvec, c, Z);
-
-  } else {
-
-    for (i=0; i<nvec; i++) {
-      Z[0]->ops->nvconst(c, Z[i]);
-    }
-    ier = 0;
-
+  }
+  else
+  {
+    for (i = 0; i < nvec; i++) { Z[0]->ops->nvconst(c, Z[i]); }
+    ier = SUN_SUCCESS;
   }
 
-  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(Z[0]));
-  return(ier);
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(Z[0]));
+  return (ier);
 }
 
-int N_VWrmsNormVectorArray(int nvec, N_Vector* X, N_Vector* W, realtype* nrm)
+SUNErrCode N_VWrmsNormVectorArray(int nvec, N_Vector* X, N_Vector* W,
+                                  sunrealtype* nrm)
 {
-  int i, ier;
+  int i;
+  SUNErrCode ier;
+
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(X[0]));
 
-  if (X[0]->ops->nvwrmsnormvectorarray != NULL) {
-
+  if (X[0]->ops->nvwrmsnormvectorarray != NULL)
+  {
     ier = X[0]->ops->nvwrmsnormvectorarray(nvec, X, W, nrm);
-
-  } else {
-
-    for (i=0; i<nvec; i++) {
-      nrm[i] = X[0]->ops->nvwrmsnorm(X[i], W[i]);
-    }
-    ier = 0;
-
+  }
+  else
+  {
+    for (i = 0; i < nvec; i++) { nrm[i] = X[0]->ops->nvwrmsnorm(X[i], W[i]); }
+    ier = SUN_SUCCESS;
   }
 
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(X[0]));
-  return(ier);
+  return (ier);
 }
 
-int N_VWrmsNormMaskVectorArray(int nvec, N_Vector* X, N_Vector* W, N_Vector id,
-                               realtype* nrm)
+SUNErrCode N_VWrmsNormMaskVectorArray(int nvec, N_Vector* X, N_Vector* W,
+                                      N_Vector id, sunrealtype* nrm)
 {
-  int i, ier;
+  int i;
+  SUNErrCode ier;
+
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(X[0]));
 
-  if (id->ops->nvwrmsnormmaskvectorarray != NULL) {
-
+  if (id->ops->nvwrmsnormmaskvectorarray != NULL)
+  {
     ier = id->ops->nvwrmsnormmaskvectorarray(nvec, X, W, id, nrm);
-
-  } else {
-
-    for (i=0; i<nvec; i++) {
+  }
+  else
+  {
+    for (i = 0; i < nvec; i++)
+    {
       nrm[i] = id->ops->nvwrmsnormmask(X[i], W[i], id);
     }
-    ier = 0;
-
+    ier = SUN_SUCCESS;
   }
 
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(X[0]));
-  return(ier);
+  return (ier);
 }
 
-int N_VScaleAddMultiVectorArray(int nvec, int nsum, realtype* a, N_Vector* X,
-                                 N_Vector** Y, N_Vector** Z)
+SUNErrCode N_VScaleAddMultiVectorArray(int nvec, int nsum, sunrealtype* a,
+                                       N_Vector* X, N_Vector** Y, N_Vector** Z)
 {
   int i, j, ier;
   N_Vector* YY = NULL;
   N_Vector* ZZ = NULL;
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(X[0]));
 
-  if (X[0]->ops->nvscaleaddmultivectorarray != NULL) {
-
+  if (X[0]->ops->nvscaleaddmultivectorarray != NULL)
+  {
     ier = X[0]->ops->nvscaleaddmultivectorarray(nvec, nsum, a, X, Y, Z);
-
-  } else if (X[0]->ops->nvscaleaddmulti != NULL ) {
-
+  }
+  else if (X[0]->ops->nvscaleaddmulti != NULL)
+  {
     /* allocate arrays of vectors */
-    YY = (N_Vector*) malloc(nsum * sizeof(N_Vector));
-    ZZ = (N_Vector*) malloc(nsum * sizeof(N_Vector));
+    YY = (N_Vector*)malloc(nsum * sizeof(N_Vector));
+    ZZ = (N_Vector*)malloc(nsum * sizeof(N_Vector));
 
-    for (i=0; i<nvec; i++) {
-
-      for (j=0; j<nsum; j++) {
+    for (i = 0; i < nvec; i++)
+    {
+      for (j = 0; j < nsum; j++)
+      {
         YY[j] = Y[j][i];
         ZZ[j] = Z[j][i];
       }
 
       ier = X[0]->ops->nvscaleaddmulti(nsum, a, X[i], YY, ZZ);
-      if (ier != 0) break;
+      if (ier != 0) { break; }
     }
 
     /* free array of vectors */
     free(YY);
     free(ZZ);
-
-  } else {
-
-    for (i=0; i<nvec; i++) {
-      for (j=0; j<nsum; j++) {
-        X[0]->ops->nvlinearsum(a[j], X[i], RCONST(1.0), Y[j][i], Z[j][i]);
+  }
+  else
+  {
+    for (i = 0; i < nvec; i++)
+    {
+      for (j = 0; j < nsum; j++)
+      {
+        X[0]->ops->nvlinearsum(a[j], X[i], SUN_RCONST(1.0), Y[j][i], Z[j][i]);
       }
     }
-    ier = 0;
+    ier = SUN_SUCCESS;
   }
 
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(X[0]));
-  return(ier);
+  return (ier);
 }
 
-int N_VLinearCombinationVectorArray(int nvec, int nsum, realtype* c,
-                                    N_Vector** X, N_Vector* Z)
+SUNErrCode N_VLinearCombinationVectorArray(int nvec, int nsum, sunrealtype* c,
+                                           N_Vector** X, N_Vector* Z)
 {
   int i, j, ier;
   N_Vector* Y = NULL;
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(X[0][0]));
 
-  if (Z[0]->ops->nvlinearcombinationvectorarray != NULL) {
-
+  if (Z[0]->ops->nvlinearcombinationvectorarray != NULL)
+  {
     ier = Z[0]->ops->nvlinearcombinationvectorarray(nvec, nsum, c, X, Z);
-
-  } else if (Z[0]->ops->nvlinearcombination != NULL ) {
-
+  }
+  else if (Z[0]->ops->nvlinearcombination != NULL)
+  {
     /* allocate array of vectors */
-    Y = (N_Vector* ) malloc(nsum * sizeof(N_Vector));
+    Y = (N_Vector*)malloc(nsum * sizeof(N_Vector));
 
-    for (i=0; i<nvec; i++) {
-
-      for (j=0; j<nsum; j++) {
-        Y[j] = X[j][i];
-      }
+    for (i = 0; i < nvec; i++)
+    {
+      for (j = 0; j < nsum; j++) { Y[j] = X[j][i]; }
 
       ier = Z[0]->ops->nvlinearcombination(nsum, c, Y, Z[i]);
-      if (ier != 0) break;
+      if (ier != 0) { break; }
     }
 
     /* free array of vectors */
     free(Y);
-
-  } else {
-
-    for (i=0; i<nvec; i++) {
+  }
+  else
+  {
+    for (i = 0; i < nvec; i++)
+    {
       Z[0]->ops->nvscale(c[0], X[0][i], Z[i]);
-      for (j=1; j<nsum; j++) {
-        Z[0]->ops->nvlinearsum(c[j], X[j][i], RCONST(1.0), Z[i], Z[i]);
+      for (j = 1; j < nsum; j++)
+      {
+        Z[0]->ops->nvlinearsum(c[j], X[j][i], SUN_RCONST(1.0), Z[i], Z[i]);
       }
     }
-    ier = 0;
+    ier = SUN_SUCCESS;
   }
 
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(X[0][0]));
-  return(ier);
+  return (ier);
 }
 
 /* -----------------------------------------------------------------
  * OPTIONAL local reduction kernels (no parallel communication)
  * -----------------------------------------------------------------*/
 
-realtype N_VDotProdLocal(N_Vector x, N_Vector y)
+sunrealtype N_VDotProdLocal(N_Vector x, N_Vector y)
 {
-  realtype result;
+  sunrealtype result;
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
-  result = ((realtype) y->ops->nvdotprodlocal(x, y));
+  result = ((sunrealtype)y->ops->nvdotprodlocal(x, y));
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
-  return(result);
+  return (result);
 }
 
-realtype N_VMaxNormLocal(N_Vector x)
+sunrealtype N_VMaxNormLocal(N_Vector x)
 {
-  realtype result;
+  sunrealtype result;
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
-  result = ((realtype) x->ops->nvmaxnormlocal(x));
+  result = ((sunrealtype)x->ops->nvmaxnormlocal(x));
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
-  return(result);
+  return (result);
 }
 
-realtype N_VMinLocal(N_Vector x)
+sunrealtype N_VMinLocal(N_Vector x)
 {
-  realtype result;
+  sunrealtype result;
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
-  result = ((realtype) x->ops->nvminlocal(x));
+  result = ((sunrealtype)x->ops->nvminlocal(x));
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
-  return(result);
+  return (result);
 }
 
-realtype N_VL1NormLocal(N_Vector x)
+sunrealtype N_VL1NormLocal(N_Vector x)
 {
-  realtype result;
+  sunrealtype result;
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
-  result = ((realtype) x->ops->nvl1normlocal(x));
+  result = ((sunrealtype)x->ops->nvl1normlocal(x));
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
-  return(result);
+  return (result);
 }
 
-realtype N_VWSqrSumLocal(N_Vector x, N_Vector w)
+sunrealtype N_VWSqrSumLocal(N_Vector x, N_Vector w)
 {
-  realtype result;
+  sunrealtype result;
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
-  result = ((realtype) x->ops->nvwsqrsumlocal(x,w));
+  result = ((sunrealtype)x->ops->nvwsqrsumlocal(x, w));
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
-  return(result);
+  return (result);
 }
 
-realtype N_VWSqrSumMaskLocal(N_Vector x, N_Vector w, N_Vector id)
+sunrealtype N_VWSqrSumMaskLocal(N_Vector x, N_Vector w, N_Vector id)
 {
-  realtype result;
+  sunrealtype result;
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
-  result = ((realtype) x->ops->nvwsqrsummasklocal(x,w,id));
+  result = ((sunrealtype)x->ops->nvwsqrsummasklocal(x, w, id));
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
-  return(result);
+  return (result);
 }
 
-booleantype N_VInvTestLocal(N_Vector x, N_Vector z)
+sunbooleantype N_VInvTestLocal(N_Vector x, N_Vector z)
 {
-  booleantype result;
+  sunbooleantype result;
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
-  result = ((booleantype) z->ops->nvinvtestlocal(x,z));
+  result = ((sunbooleantype)z->ops->nvinvtestlocal(x, z));
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
-  return(result);
+  return (result);
 }
 
-booleantype N_VConstrMaskLocal(N_Vector c, N_Vector x, N_Vector m)
+sunbooleantype N_VConstrMaskLocal(N_Vector c, N_Vector x, N_Vector m)
 {
-  booleantype result;
+  sunbooleantype result;
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
-  result = ((booleantype) x->ops->nvconstrmasklocal(c,x,m));
+  result = ((sunbooleantype)x->ops->nvconstrmasklocal(c, x, m));
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
-  return(result);
+  return (result);
 }
 
-realtype N_VMinQuotientLocal(N_Vector num, N_Vector denom)
+sunrealtype N_VMinQuotientLocal(N_Vector num, N_Vector denom)
 {
-  realtype result;
+  sunrealtype result;
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(num));
-  result = ((realtype) num->ops->nvminquotientlocal(num,denom));
+  result = ((sunrealtype)num->ops->nvminquotientlocal(num, denom));
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(num));
-  return(result);
+  return (result);
 }
 
 /* -------------------------------------------
  * OPTIONAL single buffer reduction operations
  * -------------------------------------------*/
 
-int N_VDotProdMultiLocal(int nvec, N_Vector x, N_Vector* Y, realtype* dotprods)
+SUNErrCode N_VDotProdMultiLocal(int nvec, N_Vector x, N_Vector* Y,
+                                sunrealtype* dotprods)
 {
-  int i;
+  SUNFunctionBegin(x->sunctx);
+  SUNErrCode ier = SUN_SUCCESS;
+  int i          = 0;
+
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
 
-  if (x->ops->nvdotprodmultilocal)
-    return((int) x->ops->nvdotprodmultilocal(nvec, x, Y, dotprods));
+  SUNAssert(x->ops->nvdotprodmultilocal || x->ops->nvdotprodlocal,
+            SUN_ERR_NOT_IMPLEMENTED);
 
-  if (x->ops->nvdotprodlocal) {
-    for (i = 0; i < nvec; i++) {
+  if (x->ops->nvdotprodmultilocal)
+  {
+    ier = x->ops->nvdotprodmultilocal(nvec, x, Y, dotprods);
+  }
+  else if (x->ops->nvdotprodlocal)
+  {
+    for (i = 0; i < nvec; i++)
+    {
       dotprods[i] = x->ops->nvdotprodlocal(x, Y[i]);
     }
-    return(0);
+    ier = SUN_SUCCESS;
   }
 
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
-  return(-1);
+
+  return ier;
 }
 
-int N_VDotProdMultiAllReduce(int nvec, N_Vector x, realtype* sum)
+SUNErrCode N_VDotProdMultiAllReduce(int nvec, N_Vector x, sunrealtype* sum)
 {
+  SUNFunctionBegin(x->sunctx);
+  SUNErrCode ier = SUN_SUCCESS;
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
-  if (x->ops->nvdotprodmultiallreduce)
-    return(x->ops->nvdotprodmultiallreduce(nvec, x, sum));
+  SUNAssert(x->ops->nvdotprodmultiallreduce, SUN_ERR_NOT_IMPLEMENTED);
+  ier = x->ops->nvdotprodmultiallreduce(nvec, x, sum);
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
-  return(-1);
+  return ier;
 }
 
 /* ------------------------------------
  * OPTIONAL XBraid interface operations
  * ------------------------------------*/
 
-int N_VBufSize(N_Vector x, sunindextype *size)
+SUNErrCode N_VBufSize(N_Vector x, sunindextype* size)
 {
-  int ier;
+  SUNFunctionBegin(x->sunctx);
+  SUNErrCode ier = SUN_SUCCESS;
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
-  if (x->ops->nvbufsize == NULL)
-    ier = -1;
-  else
-    ier = x->ops->nvbufsize(x, size);
+  SUNAssert(x->ops->nvbufsize, SUN_ERR_NOT_IMPLEMENTED);
+  ier = x->ops->nvbufsize(x, size);
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
-  return(ier);
+  return (ier);
 }
 
-int N_VBufPack(N_Vector x, void *buf)
+SUNErrCode N_VBufPack(N_Vector x, void* buf)
 {
-  int ier;
+  SUNFunctionBegin(x->sunctx);
+  SUNErrCode ier = SUN_SUCCESS;
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
-  if (x->ops->nvbufpack == NULL)
-    ier = -1;
-  else
-    ier = x->ops->nvbufpack(x, buf);
+  SUNAssert(x->ops->nvbufpack, SUN_ERR_NOT_IMPLEMENTED);
+  ier = x->ops->nvbufpack(x, buf);
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
-  return(ier);
+  return (ier);
 }
 
-int N_VBufUnpack(N_Vector x, void *buf)
+SUNErrCode N_VBufUnpack(N_Vector x, void* buf)
 {
-  int ier;
+  SUNFunctionBegin(x->sunctx);
+  SUNErrCode ier = SUN_SUCCESS;
   SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
-  if (x->ops->nvbufunpack == NULL)
-    ier = -1;
-  else
-    ier = x->ops->nvbufunpack(x, buf);
+  SUNAssert(x->ops->nvbufunpack, SUN_ERR_NOT_IMPLEMENTED);
+  ier = x->ops->nvbufunpack(x, buf);
   SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
-  return(ier);
+  return (ier);
 }
 
 /* -----------------------------------------------------------------
@@ -967,72 +1004,80 @@ int N_VBufUnpack(N_Vector x, void *buf)
  *   N_VCloneVectorArray
  *   N_VDestroyVectorArray
  * -----------------------------------------------------------------*/
-N_Vector* N_VNewVectorArray(int count)
+
+N_Vector* N_VNewVectorArray(int count, SUNContext sunctx)
 {
+  SUNFunctionBegin(sunctx);
   N_Vector* vs = NULL;
-
-  if (count <= 0) return(NULL);
-
-  vs = (N_Vector* ) malloc(count * sizeof(N_Vector));
-  if(vs == NULL) return(NULL);
-
-  return(vs);
+  vs           = (N_Vector*)malloc(count * sizeof(N_Vector));
+  SUNAssertNull(vs, SUN_ERR_MALLOC_FAIL);
+  return vs;
 }
 
 N_Vector* N_VCloneEmptyVectorArray(int count, N_Vector w)
 {
+  SUNFunctionBegin(w->sunctx);
   N_Vector* vs = NULL;
   int j;
 
-  if (count <= 0) return(NULL);
+  SUNAssertNull(count > 0, SUN_ERR_ARG_OUTOFRANGE);
 
-  vs = (N_Vector* ) malloc(count * sizeof(N_Vector));
-  if(vs == NULL) return(NULL);
+  vs = (N_Vector*)malloc(count * sizeof(N_Vector));
+  SUNAssertNull(vs, SUN_ERR_MALLOC_FAIL);
 
-  for (j = 0; j < count; j++) {
+  for (j = 0; j < count; j++)
+  {
     vs[j] = N_VCloneEmpty(w);
-    if (vs[j] == NULL) {
-      N_VDestroyVectorArray(vs, j-1);
-      return(NULL);
+    SUNCheckLastErrNoRet();
+    if (SUNContext_PeekLastError(SUNCTX_) < 0)
+    {
+      N_VDestroyVectorArray(vs, j - 1);
+      return (NULL);
     }
   }
 
-  return(vs);
+  return (vs);
 }
 
 N_Vector* N_VCloneVectorArray(int count, N_Vector w)
 {
-  N_Vector* vs = NULL;
+  SUNFunctionBegin(w->sunctx);
   int j;
+  N_Vector* vs = NULL;
 
-  if (count <= 0) return(NULL);
+  SUNAssertNull(count > 0, SUN_ERR_ARG_OUTOFRANGE);
 
-  vs = (N_Vector* ) malloc(count * sizeof(N_Vector));
-  if (vs == NULL) return(NULL);
+  vs = (N_Vector*)malloc(count * sizeof(N_Vector));
+  SUNAssertNull(vs, SUN_ERR_MALLOC_FAIL);
 
-  for (j = 0; j < count; j++) {
+  for (j = 0; j < count; j++)
+  {
     vs[j] = N_VClone(w);
-    if (vs[j] == NULL) {
-      N_VDestroyVectorArray(vs, j-1);
-      return(NULL);
+    SUNCheckLastErrNoRet();
+    if (SUNContext_PeekLastError(SUNCTX_) < 0)
+    {
+      N_VDestroyVectorArray(vs, j - 1);
+      return (NULL);
     }
   }
 
-  return(vs);
+  return (vs);
 }
 
 void N_VDestroyVectorArray(N_Vector* vs, int count)
 {
   int j;
 
-  if (vs == NULL) return;
+  if (vs == NULL) { return; }
 
-  for (j = 0; j < count; j++) {
+  for (j = 0; j < count; j++)
+  {
     N_VDestroy(vs[j]);
     vs[j] = NULL;
   }
 
-  free(vs); vs = NULL;
+  free(vs);
+  vs = NULL;
 
   return;
 }
@@ -1040,44 +1085,38 @@ void N_VDestroyVectorArray(N_Vector* vs, int count)
 /* These function are really only for users of the Fortran interface */
 N_Vector N_VGetVecAtIndexVectorArray(N_Vector* vs, int index)
 {
-  if (vs==NULL)       return NULL;
-  else if (index < 0) return NULL;
-  else                return vs[index];
+  SUNFunctionBegin(vs[0]->sunctx);
+  SUNAssertNull(index >= 0, SUN_ERR_ARG_OUTOFRANGE);
+  return vs[index];
 }
 
 void N_VSetVecAtIndexVectorArray(N_Vector* vs, int index, N_Vector w)
 {
-  if (vs==NULL)       return;
-  else if (index < 0) return;
-  else                vs[index] = w;
+  SUNFunctionBegin(w->sunctx);
+  SUNAssertVoid(index >= 0, SUN_ERR_ARG_OUTOFRANGE);
+  vs[index] = w;
 }
 
-
 /* -----------------------------------------------------------------
  * Debugging functions
  * ----------------------------------------------------------------- */
 
 void N_VPrint(N_Vector v)
 {
-  if (v == NULL) {
-    printf("NULL Vector\n");
-  } else if (v->ops->nvprint == NULL) {
-    printf("NULL Print Op\n");
-  } else {
-    v->ops->nvprint(v);
-  }
+  if (v == NULL) { Rprintf("%s", "NULL Vector\n"); }
+  else if (v->ops->nvprint == NULL) { Rprintf("%s", "NULL Print Op\n"); }
+  else { v->ops->nvprint(v); }
 }
 
-
 void N_VPrintFile(N_Vector v, FILE* outfile)
 {
-  if (outfile != NULL) {
-    if (v == NULL) {
-      fprintf(outfile, "NULL Vector\n");
-    } else if (v->ops->nvprintfile == NULL) {
+  if (outfile != NULL)
+  {
+    if (v == NULL) { fprintf(outfile, "NULL Vector\n"); }
+    else if (v->ops->nvprintfile == NULL)
+    {
       fprintf(outfile, "NULL PrintFile Op\n");
-    } else {
-      v->ops->nvprintfile(v, outfile);
     }
+    else { v->ops->nvprintfile(v, outfile); }
   }
 }
